/*********************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/
/********************************************************************
*
*  Filename:   EEPROM.C
*  Purpose:    This file contains Data EEPROM erase/program/read
*              functions.
*
*********************************************************************/

/* header files */
#include "eeprom.h"          /* defines eeprom macros */
#include <p30fxxxx.h>      /* processor defines */

/* funtion prototypes */
void EraseEE(void);
void EraseEEWord (unsigned EE_Addr);
void EraseEERow (unsigned EE_Row_Addr);
void ProgramEEWord (unsigned EE_Data,unsigned EE_Addr);
void ProgramEERow (unsigned Row_Addr, unsigned *Row_Data);
unsigned ReadEE (unsigned EE_Addr);


/* source code */
/*************************************************************************
*                                                                         
*  Function:  EraseEE()
*  Arguments: none
*  Returns:   void
* 
*  This function erases the entire EEPROM using the erase Row with loop.
*
**************************************************************************/
void EraseEE (void)   
{
	int	EraseLoop ;

	NVMCON = EE_ERS_ROW;			// set the NVMCON for erasing one row command
   	TBLPAG = DATA_EEPROM_TBLPAG;  	//0x7F into the NVMDRU (b16 ~ b23), address at (0x7F FC00 ~ 0x7F FFFE)
   	NVMADR = 0xFC00;  				// any EEPROM address okay
 	for ( EraseLoop = 0 ; EraseLoop < 32 ; EraseLoop ++ )  //	1024B / 32B = 32 Row 
 	{
   		/* start the erase */
   		UNLOCK_NVM_AND_PROGRAM; 	//marco define in the eeprom.h
									/*  asm ("push    w7");
										asm ("mov     #0x55, w7");  
    									asm ("mov     w7, NVMKEY");  
    									asm ("mov     #0xaa, w7");  
    									asm ("mov     w7, NVMKEY");  
    									asm ("bset    NVMCON, #0xF"); 
    									asm ("nop");
    									asm ("nop");
										asm ("pop     w7");
									*/ 
   		while (NVMCONbits.WR);		// wait until the erase completes 
		NVMADR += 32 ; 				//point to next row (16 words)
 	}        
}  /* end EraseEE() */

/*************************************************************************
*                                                                         
*  Function:  EraseEEWord()
*  Arguments: EE_Addr, the Address content to be erase
*  Returns:   void
*
*  This function programs the specified EEPROM Address to be Erase.
*
**************************************************************************/

void EraseEEWord (unsigned EE_Addr)   
{
    NVMCON = EE_ERS_WORD;			// set the NVMCON for the signal word erase command
    TBLPAG = DATA_EEPROM_TBLPAG;  	//0x7F into the NVMDRU (b16 ~ b23)
    NVMADR = EE_Addr;				// set the EEPROM address to desired word to Erase 
   	UNLOCK_NVM_AND_PROGRAM; 		//marco define in the eeprom.h
	while (NVMCONbits.WR);			// wait until the erase completes 
}

/*************************************************************************
*                                                                         
*  Function:  EraseEERow()
*  Arguments: EE_Row_Addr, the content of EEPROM Row Address to be erase
*  Returns:   void
*
*  This function programs the specified EEPROM Row Address to be Erase.
*
**************************************************************************/

void EraseEERow (unsigned EE_Row_Addr)   
{
    
    NVMCON = EE_ERS_ROW;			// set the NVMCON for a row erase command
    TBLPAG = DATA_EEPROM_TBLPAG;  	//0x7F into the NVMDRU (b16 ~ b23)
	NVMADR= EE_Row_Addr & 0xFFE0 ;	// set the EEPROM address to desired word to Erase 
   	UNLOCK_NVM_AND_PROGRAM; 		//marco define in the eeprom.h
	while (NVMCONbits.WR);			// wait until the erase completes 
}

/*************************************************************************
*                                                                         
*  Function:  ProgramEEWord()
*  Arguments: EE_Data,EE_Addr
*  Returns:   void
*
*  This function programs the specified word to the Data EEPROM.
*  The target location is the present "word_offset_address".
*
**************************************************************************/
void ProgramEEWord (unsigned EE_Data,unsigned EE_Addr)  
{ 
    NVMCON = EE_PRG_WORD;					// set the NVMCON for one word programming 
    TBLPAG = DATA_EEPROM_TBLPAG;  			// 0x7F into the NVMDRU (b16 ~ b23)

	PUSH_WREG6_WREG7;
    WREG6 = EE_Data;						// set the source register W6 to the desired word to write 
    WREG7 = (unsigned) EE_Addr; 			// set the destination pointer W7 for programming 

    asm ("tblwtl.w w6, [w7]					; load the one write latch! ");
   
    UNLOCK_NVM_AND_PROGRAM;					// now do the programming operation                      
    while (NVMCONbits.WR);					// wait until the programming completes 
 
   	POP_WREG6_WREG7;
}  /* end ProgramEEWord() */


/*************************************************************************                         
*                                                                         
*  Function:  ProgramEERow()
*  Arguments: Row_Addr, Row_Data
*  Returns:   void
*
*  This function programs one row of the Data EEPROM.
*  The data programmed is from the new_row[] array.
*  The target row in the Data EEPROM is the present "row_offset_address".
*
  **************************************************************************/
void ProgramEERow (unsigned Row_Addr, unsigned *Row_Data)   
{
    NVMCON = EE_PRG_ROW;					// set the NVMCON for the rom programming */
    TBLPAG = DATA_EEPROM_TBLPAG;			//0x7F into the NVMDRU (b16 ~ b23)

	PUSH_WREG6_WREG7;
    WREG6 = (unsigned) Row_Data; 			// set the source register W6 to the desired word to write 
    WREG7 = (unsigned) Row_Addr;			// set the destination pointer W7 for programming 

    asm ("repeat   #15             			; perform 16 writes"); \
    asm ("tblwtl.w [w6++], [w7++]  			; write one 16-bit word at a time");          
											 
    UNLOCK_NVM_AND_PROGRAM;					// now do the programming operation 
    while (NVMCONbits.WR);					// wait until the erase completes, then verify WR is still clear 

	POP_WREG6_WREG7;
   
}  /* end ProgramEERow() */


/*************************************************************************
*                                                                         
*  Function:  ReadEE()
*  Arguments: EE_Addr
*  Returns:   EE_Read_Data
*
*  This function reads the a word data from EEPROM
*
**************************************************************************/
unsigned ReadEE (unsigned EE_Addr)
{
  	unsigned EE_Read_Data;
	unsigned *ptr;

  	asm ("push 	PSVPAG") ;   					// Save the PSVPAG for other user
   	CORCONbits.PSV = 1;							// set the PSVPAG for Data EEPROM 
   	PSVPAG= 0xFF ;								// Set PSVPAG to the EEPROM location
	ptr = EE_Addr ;								// point to Offset of EEPROM
	EE_Read_Data = *ptr ;						// Read data from EEPROM
	asm ("pop 	PSVPAG") ;						// Pop PSVPAG for LCD ROM display message
	
	return EE_Read_Data;
							
}  /* end ReadEE() */

